package io.tswf.transmutations.transmute.exception

import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.meta.TransmutationMethodMetadata
import org.objectweb.asm.tree.ClassNode

@CompileStatic
class MissingTransmutationTargetMethodException extends RuntimeException implements IMissingTransmutationTargetMethodHandleExceptionBypassMarker {
    TransmutationMethodMetadata transmutationMethodMeta
    ClassNode targetClass

    MissingTransmutationTargetMethodException(TransmutationMethodMetadata transmutationMethodMeta, ClassNode targetClass) {
        super("There is no target method found in ${targetClass.javaName} for transmutation ${transmutationMethodMeta.transmutationMethod.javaFullName} in ${transmutationMethodMeta.transmutationOwner.javaName}! There is only that methods: \n${getFancyAllMethods(targetClass)}")
        this.transmutationMethodMeta = transmutationMethodMeta
        this.targetClass = targetClass
    }

    static String getFancyAllMethods(ClassNode classNode) {
        return classNode.methods
            .stream()
                .map { "asm: ${it.name}${it.desc}, fancy: ${it.javaFullName}" }
                .joinToString("\n")
    }
}
