package io.tswf.transmutations.transmute.exception

import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.meta.TransmutationMethodMetadata
import org.objectweb.asm.tree.ClassNode

@CompileStatic
class MissingTransmutationTargetMethodHandleException extends RuntimeException {
    TransmutationMethodMetadata transmutationMethodMeta
    ClassNode targetClass

    MissingTransmutationTargetMethodHandleException(TransmutationMethodMetadata transmutationMethodMeta, ClassNode targetClass) {
        super(getExceptionMessage("", transmutationMethodMeta, targetClass))
        this.transmutationMethodMeta = transmutationMethodMeta
        this.targetClass = targetClass
    }

    MissingTransmutationTargetMethodHandleException(String message, TransmutationMethodMetadata transmutationMethodMeta, ClassNode targetClass) {
        super(getExceptionMessage(getExceptionMessage(message, transmutationMethodMeta, targetClass), transmutationMethodMeta, targetClass))
        this.transmutationMethodMeta = transmutationMethodMeta
        this.targetClass = targetClass
    }

    MissingTransmutationTargetMethodHandleException(String message, Throwable cause, TransmutationMethodMetadata transmutationMethodMeta, ClassNode targetClass) {
        super(getExceptionMessage(message, transmutationMethodMeta, targetClass), cause)
        this.transmutationMethodMeta = transmutationMethodMeta
        this.targetClass = targetClass
    }

    MissingTransmutationTargetMethodHandleException(Throwable cause, TransmutationMethodMetadata transmutationMethodMeta, ClassNode targetClass) {
        super(getExceptionMessage("", transmutationMethodMeta, targetClass), cause)
        this.transmutationMethodMeta = transmutationMethodMeta
        this.targetClass = targetClass
    }

    static String getExceptionMessage(String message, TransmutationMethodMetadata transmutationMethodMeta, ClassNode targetClass) {
        return "Exception occurred while trying to handle missing target method in ${targetClass.javaName} for transmutation method ${transmutationMethodMeta.transmutationMethod.javaFullName} located in ${transmutationMethodMeta.transmutationOwner.javaName}: ${message}"
    }
}

interface IMissingTransmutationTargetMethodHandleExceptionBypassMarker {

}