package io.tswf.transmutations.transmute.injection.hook

import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.exception.InvalidTransmutationException
import io.tswf.transmutations.transmute.injection.ITransmutationPostInjectionHook
import io.tswf.transmutations.transmute.injection.contexts.TransmutationInjectionToMethodAtInsnContext
import org.objectweb.asm.Opcodes
import org.objectweb.asm.Type
import org.objectweb.asm.tree.VarInsnNode

import static org.objectweb.asm.Opcodes.*
import static io.tswf.transmutations.transmute.injection.hook.DefaultHookConstants.*

@CompileStatic
class LoadReturnValuePostInjectionHook implements ITransmutationPostInjectionHook {

    @Override
    void handleTransmutationPostInjection(TransmutationInjectionToMethodAtInsnContext context) {
        if (Opcodes.isOpcodeReturn(context.startInsn.opcode) && context.customPayload.getByKey(NEEDS_TO_LOAD_RETURN_LOCAL_VAR_KEY)) {
            def targetMethod = context.allPointsContext.targetMethod
            def targetMethodReturnType = targetMethod.returnType
            if (targetMethodReturnType == Type.VOID_TYPE) {
                throw new InvalidTransmutationException("Can't load return value from local for void method ${targetMethod.javaFullName} of ${context.allPointsContext.targetMethodOwner.javaName}")
            }

            def returnLocalVarIndex = (int) context.allPointsContext.customPayload.getByKey(RETURN_LOCAL_VAR_INDEX_KEY)
                .ensureNotNull { new IllegalStateException("There is no delayed local var index for method ${targetMethod.javaFullName} of ${context.allPointsContext.targetMethodOwner.javaName}") }

            context.insnsToInsertToCurrent << new VarInsnNode(targetMethodReturnType.getOpcode(ILOAD), returnLocalVarIndex)
        }
    }
}
