package io.tswf.transmutations.transmute.injection.parameter

import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.annotation.LocalVariableValue
import io.tswf.transmutations.transmute.exception.InvalidTransmutationException
import io.tswf.transmutations.transmute.injection.ITransmutationMethodCustomParameterLoader
import io.tswf.transmutations.transmute.injection.contexts.TransmutationInjectionToMethodAtInsnContext
import org.objectweb.asm.Type
import org.objectweb.asm.tree.FieldInsnNode
import org.objectweb.asm.tree.VarInsnNode

import java.lang.reflect.Modifier

import static org.objectweb.asm.Opcodes.*

@CompileStatic
class LoadFieldCustomParameterLoader implements ITransmutationMethodCustomParameterLoader {

    String fieldName
    Type fieldType

    LoadFieldCustomParameterLoader(String fieldName, Type fieldType) {
        this.fieldName = fieldName
        this.fieldType = fieldType
    }

    @Override
    void loadCustomParameter(TransmutationInjectionToMethodAtInsnContext context, int parameterIndex, Type parameterType) {
        def targetClass = context.allPointsContext.targetMethodOwner
        def targetMethod = context.allPointsContext.targetMethod
        def fieldToLoad = targetClass.getFieldByNameAndType(fieldName, fieldType)

        def targetFieldIsStatic = Modifier.isStatic(fieldToLoad.access)
        def targetMethodIsStatic = Modifier.isStatic(targetMethod.access)

        if (targetMethodIsStatic && !targetFieldIsStatic) {
            throw new InvalidTransmutationException("Requested field ${fieldName} of type ${fieldType.descriptor} is non-static, but target method ${targetMethod.javaFullName} of ${context.allPointsContext.targetMethodOwner.javaName} is static. Requested field can not be accessed!")
        }

        def readOpcode = Modifier.isStatic(fieldToLoad.access) ? GETSTATIC : GETFIELD

        // Для нестатического обращения к полю загружаем this
        if (!targetFieldIsStatic) {
            context.insnsToInsertToCurrent << new VarInsnNode(ALOAD, 0)
        }

        if (parameterType != fieldType) {
            // Надо оставить возможность положить сюда параметр, класс которого недоступен из класса-трансмутации
            if (parameterType == Object.asAsmType() && !fieldType.isPrimitive()) {
                // Nope
            } else {
                throw new InvalidTransmutationException("The annotated with @${LocalVariableValue.simpleName} ${parameterIndex}'st parameter of ${context.allPointsContext.transmutationMetadata.transmutationMethod.javaFullName} of ${context.allPointsContext.transmutationMetadata.transmutationOwner.javaName} type must be a ${fieldType.className}, but was ${parameterType.className}")
            }
        }

        context.insnsToInsertToCurrent << new FieldInsnNode(readOpcode, targetClass.name, fieldName, fieldType.descriptor)
    }
}
