package io.tswf.transmutations.transmute.injection.parameter

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.annotation.Self
import io.tswf.transmutations.transmute.exception.InvalidTransmutationException
import io.tswf.transmutations.transmute.injection.ITransmutationMethodCustomParameterLoader
import io.tswf.transmutations.transmute.injection.contexts.TransmutationInjectionToMethodAtInsnContext
import org.objectweb.asm.Type
import org.objectweb.asm.tree.VarInsnNode

import java.lang.reflect.Modifier

import static org.objectweb.asm.Opcodes.*

@AutoService(ITransmutationMethodCustomParameterLoader)
@CompileStatic
class SelfCustomParameterLoader implements ITransmutationMethodCustomParameterLoader {

    @Override
    void loadCustomParameter(TransmutationInjectionToMethodAtInsnContext context, int parameterIndex, Type parameterType) {
        def targetMethod = context.allPointsContext.targetMethod
        if (Modifier.isStatic(targetMethod.access)) {
            throw new InvalidTransmutationException("Can't load @${Self.simpleName} annotated parameter for static target method ${targetMethod.javaFullName} of ${context.allPointsContext.targetMethodOwner.javaName}!")
        }

        def targetClassSelfType = context.allPointsContext.targetMethodOwner.asAsmType()
        if (parameterType != Object.asAsmType() && parameterType != targetClassSelfType) {
            throw new InvalidTransmutationException("@${Self.simpleName} annotated ${parameterIndex}'st parameter of method ${context.allPointsContext.transmutationMetadata.transmutationMethod.javaFullName} of ${context.allPointsContext.transmutationMetadata.transmutationOwner.javaName} with type ${parameterType.className} does not matching target class type. It must be Object or '${targetClassSelfType.className}'!")
        }

        context.insnsToInsertToCurrent << new VarInsnNode(ALOAD, 0)
    }
}
