package io.tswf.transmutations.transmute.injection.points

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.annotation.AtFieldCall
import io.tswf.transmutations.common.reader.ClassReferenceReader
import org.objectweb.asm.Type
import org.objectweb.asm.tree.AbstractInsnNode
import org.objectweb.asm.tree.AnnotationNode
import org.objectweb.asm.tree.FieldInsnNode

import static org.objectweb.asm.Opcodes.*

@AutoService(ITransmutationInjectionPoint)
@CompileStatic
class FieldCallTransmutationInjectionPoint implements ITransmutationInjectionPoint {

    Type fieldOwner
    String fieldName
    String fieldDescriptor
    FieldCallType callType

    FieldCallTransmutationInjectionPoint(Type fieldOwner, String fieldName, String fieldDescriptor, FieldCallType callType) {
        this.fieldOwner = fieldOwner
        this.fieldName = fieldName
        this.fieldDescriptor = fieldDescriptor
        this.callType = callType
    }

    @Override
    AbstractInsnNode findNextNodeNearInject(
            InjectionPointSearchContext context
    ) {

        def insnOpcodes = callType.opcodes
        return context.insnFromSearch.asIterable().stream()
            .filter { insnCandidate ->
                if (insnCandidate instanceof FieldInsnNode) {
                    if (insnCandidate.opcode !in insnOpcodes)
                        return false

                    if (insnCandidate.name != fieldName)
                        return false

                    if (insnCandidate.desc != fieldDescriptor)
                        return false

                    if (insnCandidate.owner != fieldOwner.internalName)
                        return false

                    return true
                }

                return false
            }
            .firstOrNull()
    }
}

@CompileStatic
enum FieldCallType {
    SET([PUTFIELD, PUTSTATIC]),
    GET([GETFIELD, GETSTATIC]),
    STATIC_GET([GETSTATIC]),
    STATIC_SET([PUTSTATIC]),
    VIRTUAL_GET([GETFIELD]),
    VIRTUAL_SET([PUTFIELD]),
    ANY([])
    ;

    Set<Integer> opcodes

    FieldCallType(Collection<Integer> opcodes) {
        this.opcodes = opcodes.toSet()
    }
}