package io.tswf.transmutations.transmute.injection.points

import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.injection.reader.IInjectionPointReader

@CompileStatic
class TransmutationInjectionPointManager {

    @Lazy
    static Map<String, IInjectionPointReader> registeredInjectionPoints =
            ServiceLoader.load(IInjectionPointReader)
                    .toMapBy { it.markerAnnotationClass.asmDescriptor }

    static IInjectionPointReader findInjectionPointReaderByMarkerDesc(String markerDesc) {
        return registeredInjectionPoints[markerDesc]
    }

    static IInjectionPointReader getTransmutationByMarkerDesc(String markerDesc) {
        return findInjectionPointReaderByMarkerDesc(markerDesc).ensureNotNull { new NoSuchElementException("No injection point found for marker '${markerDesc}'!") }
    }


}
