package io.tswf.transmutations.transmute.injection.reader

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.common.reader.ClassReferenceReader
import io.tswf.transmutations.common.reader.FieldDescriptionReader
import io.tswf.transmutations.common.remapping.TransmutationRemapper
import io.tswf.transmutations.transmute.annotation.FieldValue
import io.tswf.transmutations.transmute.injection.parameter.LoadFieldCustomParameterLoader
import org.objectweb.asm.tree.AnnotationNode

@AutoService(ICustomParameterLoaderReader)
@CompileStatic
class LoadFieldCustomParameterLoaderReader implements ICustomParameterLoaderReader {

    public static final String FIELD_DESCRIPTION_PARAMETER_NAME = "value"

    @Override
    void handleLoaderSearch(CustomParameterLoaderReadingContext readingContext) {
        def fieldAnnotation = readingContext.transmutationMetadata
                .transmutationMethod
                .findParameterAnnotationByType(readingContext.parameterIndex, FieldValue)

        if (fieldAnnotation != null) {
            def (rawFieldName, rawFieldType) = FieldDescriptionReader.findFieldDescriptionFromAnnotation(
                    fieldAnnotation.getValueByName(FIELD_DESCRIPTION_PARAMETER_NAME, AnnotationNode)
            )

            def (mappedFieldName, mappedFieldType) = TransmutationRemapper.mapField(readingContext.transmutationMetadata.targetClass, rawFieldName, rawFieldType)

            readingContext.transmutationMetadata.customParameterLoaders << new LoadFieldCustomParameterLoader(mappedFieldName, mappedFieldType)
            readingContext.completed = true
        }
    }
}
