package io.tswf.transmutations.transmute.injection.reader.points

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.common.reader.ConstantReader
import io.tswf.transmutations.transmute.annotation.AtConstant
import io.tswf.transmutations.transmute.exception.InvalidTransmutationDescriptionException
import io.tswf.transmutations.transmute.injection.points.ConstantTransmutationInjectionPoint
import io.tswf.transmutations.transmute.injection.reader.IInjectionPointReader
import io.tswf.transmutations.transmute.injection.reader.InjectionPointReadingContext
import org.objectweb.asm.tree.AnnotationNode

@AutoService(IInjectionPointReader)
@CompileStatic
class ConstantTransmutationInjectionPointReader implements IInjectionPointReader {

    public static final String ATC_CONSTANT_NAME = "value"

    public static Collection<String> ALLOWED_CONSTANT_DESCRIPTORS = [
            String.asmDescriptor,
            Integer.asmDescriptor,
            Double.asmDescriptor,
            Float.asmDescriptor,
            Long.asmDescriptor
    ]

    @Override
    void handleInjectionPointReading(InjectionPointReadingContext context) {
        def requestedConstantValue = ConstantReader.getConstantFromAnnotation(
                context.annotationNodeFromRead.getValueByName(ATC_CONSTANT_NAME, AnnotationNode)
        )

        if (requestedConstantValue == null) {
            throw new InvalidTransmutationDescriptionException("Requested constant value can't be null for ${context.annotationNodeFromRead.desc}!")
        }

        if (requestedConstantValue.class.asmDescriptor !in ALLOWED_CONSTANT_DESCRIPTORS) {
            throw new InvalidTransmutationDescriptionException("Constant type '${requestedConstantValue.class.asmDescriptor}' not allowed! Please, use one of these: ${ALLOWED_CONSTANT_DESCRIPTORS}")
        }

        context.result = new ConstantTransmutationInjectionPoint(requestedConstantValue)
    }

    @Override
    Class getMarkerAnnotationClass() {
        return AtConstant
    }
}
