package io.tswf.transmutations.transmute.injection.reader.points

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.common.reader.ClassReferenceReader
import io.tswf.transmutations.common.reader.FieldDescriptionReader
import io.tswf.transmutations.common.remapping.TransmutationRemapper
import io.tswf.transmutations.transmute.annotation.AtFieldCall
import io.tswf.transmutations.transmute.injection.points.FieldCallTransmutationInjectionPoint
import io.tswf.transmutations.transmute.injection.points.FieldCallType
import io.tswf.transmutations.transmute.injection.reader.IInjectionPointReader
import io.tswf.transmutations.transmute.injection.reader.InjectionPointReadingContext
import org.objectweb.asm.tree.AnnotationNode

@AutoService(IInjectionPointReader)
@CompileStatic
class FieldCallTransmutationInjectionPointReader implements IInjectionPointReader {

    public static final String ATF_TYPE_PARAMETER_NAME = "callType"
    public static final String ATF_OWNER_CLASS_PARAMETER_NAME = "owner"
    public static final String ATF_FIELD_DESCRIPTION_PARAMETER_NAME = "field"

    @Override
    void handleInjectionPointReading(InjectionPointReadingContext context) {
        def atFieldAnnotation = context.annotationNodeFromRead
        def callType = (atFieldAnnotation.findValueByName(ATF_TYPE_PARAMETER_NAME, FieldCallType)
                ?: FieldCallType.ANY)

        def rawFieldOwner = ClassReferenceReader.getTypeFromClassRefAnnotation(
                atFieldAnnotation.getValueByName(ATF_OWNER_CLASS_PARAMETER_NAME, AnnotationNode)
        )

        def fieldDescriptionAnnotation = atFieldAnnotation.getValueByName(ATF_FIELD_DESCRIPTION_PARAMETER_NAME, AnnotationNode)

        def (rawFieldName, rawFieldType) = FieldDescriptionReader.findFieldDescriptionFromAnnotation(fieldDescriptionAnnotation)

        def mappedFieldOwner = TransmutationRemapper.mapType(rawFieldOwner)

        def (mappedFieldName, mappedFieldType) = TransmutationRemapper.mapField(rawFieldOwner, rawFieldName, rawFieldType)

        context.result = new FieldCallTransmutationInjectionPoint(mappedFieldOwner, mappedFieldName, mappedFieldType.descriptor, callType)
    }

    @Override
    Class getMarkerAnnotationClass() {
        return AtFieldCall
    }
}
