package io.tswf.transmutations.transmute.injection.reader.points

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.common.reader.ClassReferenceReader
import io.tswf.transmutations.common.remapping.TransmutationRemapper
import io.tswf.transmutations.transmute.annotation.AtLocalVariableCall
import io.tswf.transmutations.transmute.exception.InvalidTransmutationDescriptionException
import io.tswf.transmutations.transmute.injection.points.LocalVariableCallTransmutationInjectionPoint
import io.tswf.transmutations.transmute.injection.points.LocalVariableCallType
import io.tswf.transmutations.transmute.injection.reader.IInjectionPointReader
import io.tswf.transmutations.transmute.injection.reader.InjectionPointReadingContext
import org.objectweb.asm.tree.AnnotationNode

@AutoService(IInjectionPointReader)
@CompileStatic
class LocalVariableCallTransmutationInjectionPointReader implements IInjectionPointReader {

    public static final String ATLVAR_INDEX_PARAMETER_NAME = "index"
    public static final String ATLVAR_CALL_TYPE_PARAMETER_NAME = "callType"
    public static final String ATLVAR_VAR_TYPE_PARAMETER_NAME = "variableType"

    @Override
    void handleInjectionPointReading(InjectionPointReadingContext context) {
        def localVarIndex = context.annotationNodeFromRead.getValueByName(ATLVAR_INDEX_PARAMETER_NAME, Integer)
        def rawLocalVarType = ClassReferenceReader.getTypeFromClassRefAnnotation(
                context.annotationNodeFromRead.getValueByName(ATLVAR_VAR_TYPE_PARAMETER_NAME, AnnotationNode)
        )
        def localVarCallInsn = context.annotationNodeFromRead.getValueByName(ATLVAR_CALL_TYPE_PARAMETER_NAME, LocalVariableCallType)

        if (localVarIndex < 0) {
            throw new InvalidTransmutationDescriptionException("Negative local variable index found: ${localVarIndex}! It is not allowed!")
        }

        def mappedLocalVarType = TransmutationRemapper.mapType(rawLocalVarType)

        context.result = new LocalVariableCallTransmutationInjectionPoint(localVarIndex, mappedLocalVarType, localVarCallInsn)
    }

    @Override
    Class getMarkerAnnotationClass() {
        return AtLocalVariableCall
    }
}
