package io.tswf.transmutations.transmute.injection.returnvalue

import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.exception.InvalidTransmutationException
import io.tswf.transmutations.transmute.injection.ITransmutationMethodReturnValueProcessor
import io.tswf.transmutations.transmute.injection.contexts.TransmutationInjectionToMethodAtInsnContext
import org.objectweb.asm.Type
import org.objectweb.asm.tree.JumpInsnNode
import org.objectweb.asm.tree.LabelNode
import org.objectweb.asm.tree.MethodNode

import static org.objectweb.asm.Opcodes.IFEQ
import static org.objectweb.asm.Opcodes.IFNE

@CompileStatic
abstract class AbstractIfTrueTransmutationTrueReturnValueProcessor implements ITransmutationMethodReturnValueProcessor {

    @Override
    void handleTransmutationMethodReturnValue(TransmutationInjectionToMethodAtInsnContext context) {
        def targetMethod = context.allPointsContext.targetMethod

        validateTarget(targetMethod)

        def transmutationMethod = context.allPointsContext.transmutationMetadata.transmutationMethod

        if (transmutationMethod.returnType != Type.BOOLEAN_TYPE) {
            throw new InvalidTransmutationException("Transmutation method ${transmutationMethod.javaFullName} of ${context.allPointsContext.transmutationMetadata.transmutationOwner.javaName} must has boolean return type for constant returning!")
        }

        def labelIfFalse = new LabelNode()

        context.insnsToInsertToCurrent << new JumpInsnNode(IFEQ, labelIfFalse)
        addReturnInsns(context)
        context.insnsToInsertToCurrent << labelIfFalse
    }

    abstract void validateTarget(MethodNode methodNode)

    abstract void addReturnInsns(TransmutationInjectionToMethodAtInsnContext context);
}

