package io.tswf.transmutations.transmute.util

import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.annotation.At
import io.tswf.transmutations.transmute.exception.InvalidAtAnnotationException
import io.tswf.transmutations.transmute.injection.contexts.InjectionOffset
import io.tswf.transmutations.transmute.injection.points.InjectionPointSearchContext
import io.tswf.transmutations.transmute.injection.points.TransmutationInjectionPointManager
import io.tswf.transmutations.transmute.injection.reader.InjectionPointReadingContext
import io.tswf.transmutations.transmute.meta.TransmutationMethodInjectionPointMetadata
import org.objectweb.asm.tree.AnnotationNode

@CompileStatic
class AtUtil {

    public static final Collection<String> AT_ANNOTATION_PARAMETER_NAMES = [
            "atConstant",
            "atExit",
            "atField",
            "atHead",
            "atLine",
            "atLocalVariable",
            "atMethod",
            "atReturn",
            "atThrow"
    ]

    static TransmutationMethodInjectionPointMetadata getInjectionPointMetadataFromAnnotation(AnnotationNode atAnnotation) {
        def foundParameters = AT_ANNOTATION_PARAMETER_NAMES.toMap(
                new HashMap<String, AnnotationNode>(),
                { it },
                { atAnnotation.findValueByName(it, AnnotationNode) }
        )

        foundParameters.removeAll { it.value == null }

        if (foundParameters.isEmpty()) {
            throw new InvalidAtAnnotationException("There is no 'at' attribute found in ${At.class.name} annotation! Please, select one from ${AT_ANNOTATION_PARAMETER_NAMES}")
        }

        if (foundParameters.size() > 1) {
            throw new InvalidAtAnnotationException("There is multiple 'at' attributes found in ${At.class.name} annotation: ${foundParameters.keySet()}. Please, choose single!")
        }

        def injectionPointReader = TransmutationInjectionPointManager.getTransmutationByMarkerDesc(
                foundParameters.values().first().desc
        )

        def injectionPointAnnotation = foundParameters.values().first()

        def injectionPointReadingContext = new InjectionPointReadingContext(annotationNodeFromRead: injectionPointAnnotation)
        injectionPointReader.handleInjectionPointReading(injectionPointReadingContext)
        def injectionPoint = injectionPointReadingContext.result

        def offset = atAnnotation.findValueByName("offset", AnnotationNode)
                ?.with { it.getValueByName("value", InjectionOffset) }
                ?: InjectionOffset.DEFAULT

        def skip = atAnnotation.findValueByName("skip", Integer)
            ?: 0

        return new TransmutationMethodInjectionPointMetadata(
                injectionOffset: offset,
                injectionPoint: injectionPoint,
                skip: skip
        )
    }
}
